import cv2 as cv
import numpy as np
import tkinter as tk
from tkinter import filedialog, messagebox
import os
from PIL import Image, ImageTk
import configparser
from SharedData import SharedData
from Commands import Commands

class GUI:
    ## 
    # @brief This class creates a graphical user interface for the image projector.
    # The GUI class is responsible for initializing the main window and setting up the control window. It provides a canvas for displaying images and buttons for loading and displaying images. 
    def __init__(self, master):
        ##
        # @brief Constructor for the GUI class.
        # This constructor initializes the main window and sets up the control window.
        #
        # @param master the parent window
        # @param self the object pointer
        # @return None
        self.master = master
        self.master.title("Image Projector")
        self.canvas = tk.Canvas(self.master, bg="black")
        self.canvas.pack(fill=tk.BOTH, expand=True)
        self.shared_data = SharedData()
        self.commands = Commands(self, self.shared_data)
        self.control_window()

    def control_window(self):
        ##
        # @brief Call the function to set up the control window.
        # A seperate window which allows loading and setting up images and overlap region. It contains buttons and entry
        # fields for loading images, setting the overlap region and control the display of images.
        #
        # @param self the object pointer

        self.control_window = tk.Toplevel(self.master)
        self.control_window.title("Controls")
        
        self.overlap = tk.IntVar(value=200)

        # Label and entry for overlap region width
        self.overlapLabel = tk.Label(self.control_window, text='Overlap region Width', font=('calibre', 10, 'bold'))
        self.overlapBox = tk.Entry(self.control_window, textvariable=self.overlap)
        
        self.overlapLabel.grid(row=0, column=0, padx=5, pady=5)
        self.overlapBox.grid(row=0, column=1, padx=5, pady=5)

        # Button to set overlap region
        self.set_overlap_button = tk.Button(self.control_window, text="Set Overlap Region", command=self.commands.set_overlap_region)
        self.set_overlap_button.grid(row=1, column=0, columnspan=2, padx=5, pady=5)

        # New buttons for increasing and decreasing the overlap region
        self.increase_overlap_button = tk.Button(self.control_window, text="Increase Overlap", command=self.commands.increase_overlap)
        self.increase_overlap_button.grid(row=2, column=0, padx=5, pady=5)

        self.decrease_overlap_button = tk.Button(self.control_window, text="Decrease Overlap", command=self.commands.decrease_overlap)
        self.decrease_overlap_button.grid(row=2, column=1, padx=5, pady=5)

        # Load left and right image buttons
        self.button_load_left = tk.Button(self.control_window, text="Load Left Image", command=self.commands.load_left_image)
        self.button_load_left.grid(row=3, column=0, padx=5, pady=5)

        self.button_load_right = tk.Button(self.control_window, text="Load Right Image", command=self.commands.load_right_image)
        self.button_load_right.grid(row=3, column=1, padx=5, pady=5)

        # Show left and right image buttons
        self.button_show_left = tk.Button(self.control_window, text="Show Left Image", command=self.commands.show_left_image)
        self.button_show_left.grid(row=4, column=0, padx=5, pady=5)

        self.button_show_right = tk.Button(self.control_window, text="Show Right Image", command=self.commands.show_right_image)
        self.button_show_right.grid(row=4, column=1, padx=5, pady=5)

        # Show original left and right image buttons
        self.button_show_left_original = tk.Button(self.control_window, text="Show Left Original", command=self.commands.show_left_original)
        self.button_show_left_original.grid(row=5, column=0, padx=5, pady=5)

        self.button_show_right_original = tk.Button(self.control_window, text="Show Right Original", command=self.commands.show_right_original)
        self.button_show_right_original.grid(row=5, column=1, padx=5, pady=5)

        self.save_config = tk.Button(self.control_window, text="Save Config", command=self.shared_data.write_to_config)
        self.save_config.grid(row=6, column=0, padx=5, pady=5)

        self.load_config = tk.Button(self.control_window, text="Load Config", command=self.shared_data.read_from_config)
        self.load_config.grid(row=6, column=1, padx=5, pady=5)


 