import cv2 as cv
import numpy as np
import os
from PIL import Image
import tkinter as tk

class ImageIO:
    ## @brief This class is in charge of image input and output
    def __init__(self, shared_data):
        ## 
        # @brief Initializator for whole method.
        # Get parsed config file data from the class 'SharedData'
        # @param shared_dara parsed config file
        # @return None
        #
        # @warning This method will not check config data is valid or not
        self.shared_data = shared_data

    def load_image(self, path):
        ##
        # @brief Function for loading image
        # @param path Path of the image
        # @return numpy array, image
        #
        # @warning This method does not check the file format or image content
        return cv.imread(path, cv.IMREAD_COLOR)

    def write_image(self, img, fileName="image.png"):
        ## 
        # @brief Function to manipulate the image
        # @param img Loaded image, numpy matrix
        # @param fileName Name of the image. "image.png" will be the default vale if there are no input
        # @return None
        if cv.imwrite(fileName, img):
            print(f"File {fileName} Written Successfully")
        else:
            print("File writing failed")

    def show_image(self, img, title="Image Title", canvas=None):
        ## 
        # @brief Function to print image through the projector
        # @param img Numpy array, loaded image
        # @param title Title of the window. 'Image Title' will be the default value if there is no input.
        # @paran Canvas parameter needed for library 'Tinker Canvas'. None is default value.
        # @return None

        # Set up OpenCV fullscreen window in the main thread
        cv.namedWindow(title, cv.WINDOW_NORMAL)
        cv.setWindowProperty(title, cv.WND_PROP_FULLSCREEN, cv.WINDOW_FULLSCREEN)
        
        # Resize the image to 1920x1080 resolution
        img_resized = cv.resize(img, (1920, 1080), interpolation=cv.INTER_LINEAR)
        
        # Display the resized image in OpenCV and keep it open without blocking Tkinter
        cv.imshow(title, img_resized)

        # Run a non-blocking loop to keep OpenCV window and Tkinter GUI responsive
        while cv.getWindowProperty(title, cv.WND_PROP_VISIBLE) >= 1:
            cv.waitKey(1)
            if canvas:
                canvas.update_idletasks()
                canvas.update()

        # Close the OpenCV window once the loop ends
        cv.destroyAllWindows()